<?php

namespace Drupal\scorm_field\Plugin\rest\resource;

use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\scorm_field\ScormFieldCommonService;
use Drupal\Core\Url;
use Drupal\user\UserInterface;

/**
 * Scorm decoupled access token resource.
 *
 * @RestResource (
 *   id = "scorm_decoupled_create_attempt_resource",
 *   label = @Translation("Scorm Decoupled Create Attempt Resource"),
 *   uri_paths = {
 *     "create" = "/api/scorm-field-scorm-create-attempt",
 *   }
 * )
 */
class ScormDecoupledCreateAttemptResource extends ResourceBase {

  /**
   * The key-value storage.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueStoreInterface
   */
  protected $storage;

  /**
   * ScormFieldCommonService
   */
  protected $scormFieldCommonService;  

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    KeyValueFactoryInterface $keyValueFactory,
    ScormFieldCommonService $scorm_field_common_service
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger, $keyValueFactory, $scorm_field_common_service);
    $this->storage = $keyValueFactory->get('scorm_field_scorm_report');
    $this->scormFieldCommonService = $scorm_field_common_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('keyvalue'),
      $container->get('scorm_field.common_service')
    );
  }

    /**
   * Responds to POST requests and saves the new record.
   */
  public function post(array $data): ModifiedResourceResponse {

    $response = [];

    try {

      if (isset($data['token']) && empty($data['token'])) {
        throw new BadRequestHttpException("The client has not defined the access token.");
      }

      if (isset($data['node_id']) && empty($data['node_id'])) {
        throw new BadRequestHttpException("The client has not defined the node id.");
      }

      if ($user_id = $data['user_id']) {
        $user = \Drupal::service('entity.repository')->loadEntityByUuid('user', $user_id);
        if ($user instanceof UserInterface) {
          $current_user = $user->uuid();
          $current_user_id = $user->id();
        }      
      }
      else {
        $anon_user_id = \Drupal::currentUser()->id();
        $user = \Drupal::entityTypeManager()->getStorage('user')->load($anon_user_id);  
        $current_user = $user->uuid();
        $current_user_id = $user->id();       
      }

      $token_valid = $this->scormFieldCommonService->validateToken($data['token'], $data['user_id']);

      if ($token_valid) { 

        // When token valid we want to generate an access token and save it into the DB
        $generated_token = $this->scormFieldCommonService->generateToken();

        $this->scormFieldCommonService->saveTokenIntoDB($generated_token);
        // Load the node
        $entity = \Drupal::entityTypeManager()->getStorage('node')->load($data['node_id']);
        // Attempt setting
        $attempt_factory = \Drupal::service('attempt_mgmt.factory');

        // Create Attempt
        $attempt_uuid = $attempt_factory->createAttempt($entity, $current_user_id);        

        if (!isset($attempt_uuid)) {
          throw new BadRequestHttpException("The attempt uuid can't be empty!");
        }
        
        $options = [
          'absolute' => TRUE,
          'query' => ['uid' => $current_user, 'attempt_uuid' => $attempt_uuid, 'attempt' => 'new']
        ];
        
        $decoupled_url = Url::fromRoute('scorm_field.decoupled', ['node' => $data['node_id'], 'token' => $generated_token], $options);
        
        $response['status'] = 'success';
        $response['action'] = 'token generated';
        $response['generated_token'] = $generated_token;
        $response['attempt_uuid'] = $attempt_uuid;          
        $response['iframe_source'] = $decoupled_url->toString();
                
        $response = new ModifiedResourceResponse($response, 201);     
        
        return $response;

      }


    }
    catch(BadRequestHttpException $e) {

      $this->logger->warning($e->getMessage());
      $error['error'] = $e->getMessage();
      $response = new ModifiedResourceResponse($error, 400);      

    }  
    catch(EntityStorageException $e) {

      $this->logger->warning($e->getMessage());
      $error['error'] = $this->helper->getErrorMessage('EntityStorageException');
      $response = new ModifiedResourceResponse($error, 409);   

    }
    catch(\Exception $e) {

      $this->logger->warning($e->getMessage());
      $error['error'] = $e->getMessage();
      $response = new ModifiedResourceResponse($error, 400);

    }   

    return $response;

  }

 
}
